/*********************************************************************/
/* iso6.c       v 2.0      by J.Sobczyk IAPW       1997.08.28  12:00 */
/*                                                                   */
/* Polish document converter.                                        */
/* ISO 6937-2                                                        */
/*********************************************************************/
#include <stdio.h>
#include "polcnv.h"

/*-------------------------------------------------------------------*/
unicode iso_6937_2_tbl[2*POL_NR-2][2] = /* ISO 6937-2 */
  {		/* Two byte codes - high order byte first */
    { 0x061CE,  0x0105 },   /* LATIN SMALL LETTER A WITH OGONEK */
    { 0x063C2,  0x0107 },   /* LATIN SMALL LETTER C WITH ACUTE */
    { 0x065CE,  0x0119 },   /* LATIN SMALL LETTER E WITH OGONEK */
    { 0x06EC2,  0x0144 },   /* LATIN SMALL LETTER N WITH ACUTE */
    { 0x06FC2,  0x00F3 },   /* LATIN SMALL LETTER O WITH ACUTE */
    { 0x073C2,  0x015B },   /* LATIN SMALL LETTER S WITH ACUTE */
    { 0x07AC2,  0x017A },   /* LATIN SMALL LETTER Z WITH ACUTE */
    { 0x07AC7,  0x017C },   /* LATIN SMALL LETTER Z WITH DOT ABOVE */
    { 0x041CE,  0x0104 },   /* LATIN CAPITAL LETTER A WITH OGONEK */
    { 0x043C2,  0x0106 },   /* LATIN CAPITAL LETTER C WITH ACUTE */
    { 0x045CE,  0x0118 },   /* LATIN CAPITAL LETTER E WITH OGONEK */
    { 0x04EC2,  0x0143 },   /* LATIN CAPITAL LETTER N WITH ACUTE */
    { 0x04FC2,  0x00D3 },   /* LATIN CAPITAL LETTER O WITH ACUTE */
    { 0x054C2,  0x015A },   /* LATIN CAPITAL LETTER S WITH ACUTE */
    { 0x05AC2,  0x0179 },   /* LATIN CAPITAL LETTER Z WITH ACUTE */
    { 0x05AC7,  0x017B }    /* LATIN CAPITAL LETTER Z WITH DOT ABOVE */
  };

/*-------------------------------------------------------------------*/
unicode iso_6937_2_tbx[2][2] = /* ISO 6937-2 */
  {		/* Single byte codes */
    { 0x0F8,  0x0142 },   /* LATIN SMALL LETTER L WITH STROKE */
    { 0x0E8,  0x0141 }    /* LATIN CAPITAL LETTER L WITH STROKE */
  };

long int iso_6937_2_hits  = 0;
long int iso_6937_2_fails = 0;

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void iso_6937_2_decode( void )
  {
  int i;
  unicode ch0;
  unicode ch1;
  int limit;
 
  limit = inp_ffree;
  if (!inp_eof)
    limit--;
 
  while ((inp_fused < limit) &&
         (uni_ffree < uni_size ))
    {
    ch0 = (unicode) inp_buf[ inp_fused++ ];
    ch1 = (ch0 & 0x0FF) << 8;
    if (inp_fused < inp_ffree)
      ch1 |= (unicode) inp_buf[ inp_fused ] & 0x0FF;
    else
      ch1 |= 0x0FF;

    if      (ch0 == iso_6937_2_tbx[0][0])
      ch0 = iso_6937_2_tbx[0][1];
    else if (ch0 == iso_6937_2_tbx[1][0])
      ch0 = iso_6937_2_tbx[1][1];
    else
      {
      for (i=0; i< 2*POL_NR-2; i++)
	if (ch1 == iso_6937_2_tbl[i][0])
	  {
	  inp_fused++;
	  ch0 = iso_6937_2_tbl[i][1];
	  break;
	  }
      }
    uni_buf[ uni_ffree++ ] = ch0;
    }
 
  if ((inp_ffree == inp_fused) && inp_eof)
    uni_eof = TRUE;
  }
 
/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void iso_6937_2_encode( void )
  {
  int i;
  unicode ch;
 
  while ((uni_ffree > uni_fused) &&
         (out_ffree < out_size-1))
    {
    ch = uni_buf[uni_fused++];
 
    if      (ch == iso_6937_2_tbx[0][1])
      ch = iso_6937_2_tbx[0][0];
    else if (ch == iso_6937_2_tbx[1][1])
      ch = iso_6937_2_tbx[1][0];
    else
      {
      for (i=0; i< 2*POL_NR-2; i++)
	if (ch == iso_6937_2_tbl[i][1])
	  {
	  ch = (iso_6937_2_tbl[i][0] >> 8) & 0x0FF;
	  out_buf[out_ffree++] = (byte) ch;
	  ch =  iso_6937_2_tbl[i][0]       & 0x0FF;
	  break;
	  }
      }
 
    out_buf[out_ffree++] = (byte) ch;
    }
 
  if ((uni_ffree == uni_fused) && uni_eof)
    out_eof = TRUE;
  }
 
/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void test_char_iso_6937_2( char *buff )
  {
  int i;

  for (i=0; i< 2; i++)
    if (buff[0] == iso_6937_2_tbx[i][0])
      {
      iso_6937_2_hits++;
      return;
      }
      
  for (i=0; i< 2*POL_NR-2; i++)
    if (((char) buff[0] == iso_6937_2_tbl[i][0]) &&
        ((char) buff[1] == iso_6937_2_tbl[i][1])   )
      {
      iso_6937_2_hits++;
      return;
      }

  if (buff[0] & 0x80)
    iso_6937_2_fails++;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int test_freq_iso_6937_2( void )
  {
  return iso_6937_2_hits-iso_6937_2_fails;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init_iso_6937_2( void )
  {
  trans[ TYPE_ISO_6937_2 ].key		= "-i6";
  trans[ TYPE_ISO_6937_2 ].long_key	= "-iso-6937-2";
  trans[ TYPE_ISO_6937_2 ].name		= "ISO 6937-2";
  trans[ TYPE_ISO_6937_2 ].reset_inp	= NULL;;
  trans[ TYPE_ISO_6937_2 ].reset_out	= NULL;;
  trans[ TYPE_ISO_6937_2 ].decode	= iso_6937_2_decode;
  trans[ TYPE_ISO_6937_2 ].encode	= iso_6937_2_encode;
  trans[ TYPE_ISO_6937_2 ].inp_tbl	= NULL;
  trans[ TYPE_ISO_6937_2 ].inp_tbl_size	= 0;
  trans[ TYPE_ISO_6937_2 ].out_tbl	= NULL;
  trans[ TYPE_ISO_6937_2 ].out_tbl_size	= 0;
  trans[ TYPE_ISO_6937_2 ].buf_tst	= NULL;
/*  trans[ TYPE_ISO_6937_2 ].char_tst	= test_char_iso_6937_2;*/
  trans[ TYPE_ISO_6937_2 ].freq_tst	= test_freq_iso_6937_2;
  }

/*********************************************************************/
