/*********************************************************************/
/* java.c       v 2.0      by J.Sobczyk IAPW       1997.10.04  19:00 */
/*                                                                   */
/* Polish document converter.                                        */
/* UNICODE for Java                                                  */
/*********************************************************************/
#include <stdio.h>
#include <ctype.h>
#include "polcnv.h"

long int java_hits  = 0;
long int java_fails = 0;

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
unicode java_hex( int c0, int c1, int c2, int c3 )
  {
  if (isdigit(c0))
    c0 = c0 - '0';
  else
    c0 = tolower(c0) - 'a' + 10;

  if (isdigit(c1))
    c1 = c1 - '0';
  else
    c1 = tolower(c1) - 'a' + 10;

  if (isdigit(c2))
    c2 = c2 - '0';
  else
    c2 = tolower(c2) - 'a' + 10;

  if (isdigit(c3))
    c3 = c3 - '0';
  else
    c3 = tolower(c3) - 'a' + 10;

  return (c0 << 12) + (c1 << 8) + (c2 << 4) + c3;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void java_decode( void )
  {
  int limit;

  limit = inp_ffree;
  if (!inp_eof)
    limit -= 6;

  while ((inp_fused < limit) &&
	 (uni_ffree < uni_size ))
    {
    if ((inp_buf[inp_fused] == '\\') && (inp_buf[inp_fused+1] == 'u') )
      {
      uni_buf[ uni_ffree++ ] = java_hex(  inp_buf[inp_fused+2],
					  inp_buf[inp_fused+3],
					  inp_buf[inp_fused+4],
					  inp_buf[inp_fused+5] );
      inp_fused += 6;
      }
    else
      uni_buf[ uni_ffree++ ] = (unicode)(inp_buf[ inp_fused++ ]);
    }

  if ((inp_ffree == inp_fused) && inp_eof)
    uni_eof = TRUE;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
byte hex_digit( unicode n )
  {
  if (n < 10)
    return n + '0';
  else
    return n - 10 + 'a';
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void java_encode( void )
  {
  unicode ch;
  unicode res;

  while ((uni_ffree > uni_fused) &&
	 (out_ffree < out_size-1))
    {
    if (uni_buf[uni_fused] < 0x07FUL)
      {
      out_buf[out_ffree++] = (byte) uni_buf[uni_fused];
      }
    else if (uni_buf[uni_fused] < 0x0FFFFUL)
      {
      out_buf[out_ffree++] = '\\';
      out_buf[out_ffree++] = 'u';
      out_buf[out_ffree++] = hex_digit((uni_buf[uni_fused] >> 12)  & 0x0F);
      out_buf[out_ffree++] = hex_digit((uni_buf[uni_fused] >>  8)  & 0x0F);
      out_buf[out_ffree++] = hex_digit((uni_buf[uni_fused] >>  4)  & 0x0F);
      out_buf[out_ffree++] = hex_digit( uni_buf[uni_fused]         & 0x0F);
      }
    else if (uni_buf[uni_fused] > 0x10FFFFUL)
      {
      out_buf[out_ffree++] = '\\';
      out_buf[out_ffree++] = 'u';
      out_buf[out_ffree++] = 'f';
      out_buf[out_ffree++] = 'f';
      out_buf[out_ffree++] = 'f';
      out_buf[out_ffree++] = 'd';
      }

    else
      {
      ch = uni_buf[uni_fused] - 0x10000UL;

      res = (ch >> 10) + 0x0D800UL;
      out_buf[out_ffree++] = '\\';
      out_buf[out_ffree++] = 'u';
      out_buf[out_ffree++] = hex_digit((res >> 12)  & 0x0F);
      out_buf[out_ffree++] = hex_digit((res >>  8)  & 0x0F);
      out_buf[out_ffree++] = hex_digit((res >>  4)  & 0x0F);
      out_buf[out_ffree++] = hex_digit( res         & 0x0F);

      res = (ch & 0x03FFUL) + 0x0DC00UL;
      out_buf[out_ffree++] = '\\';
      out_buf[out_ffree++] = 'u';
      out_buf[out_ffree++] = hex_digit((res >> 12)  & 0x0F);
      out_buf[out_ffree++] = hex_digit((res >>  8)  & 0x0F);
      out_buf[out_ffree++] = hex_digit((res >>  4)  & 0x0F);
      out_buf[out_ffree++] = hex_digit( res         & 0x0F);
      }

    uni_fused++;
    }

  if ((uni_ffree == uni_fused) && uni_eof)
    out_eof = TRUE;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void test_char_java( char *buff )
  {
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int test_freq_java( void )
  {
  return java_hits-java_fails;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init_unicode_java( void )
  {
  trans[ TYPE_UNICODE_JAVA ].key		= "-j";
  trans[ TYPE_UNICODE_JAVA ].long_key		= "-java";
  trans[ TYPE_UNICODE_JAVA ].name		= "Unicode UCS-2 for Java";
  trans[ TYPE_UNICODE_JAVA ].reset_inp		= NULL;
  trans[ TYPE_UNICODE_JAVA ].reset_out		= NULL;
  trans[ TYPE_UNICODE_JAVA ].decode		= java_decode;
  trans[ TYPE_UNICODE_JAVA ].encode		= java_encode;
  trans[ TYPE_UNICODE_JAVA ].inp_tbl		= NULL;
  trans[ TYPE_UNICODE_JAVA ].inp_tbl_size	= 0;
  trans[ TYPE_UNICODE_JAVA ].out_tbl		= NULL;
  trans[ TYPE_UNICODE_JAVA ].out_tbl_size	= 0;
  trans[ TYPE_UNICODE_JAVA ].buf_tst		= NULL;
  trans[ TYPE_UNICODE_JAVA ].freq_tst		= test_freq_java;
  }

/*********************************************************************/
