/*********************************************************************/
/* utf7.c    v 2.0      by J.Sobczyk IAPW       1997.09.04  21:00    */
/*                                                                   */
/* Polish document converter.                                        */
/* Unicode (ISO 10646) UTF7 as described in RFC 2152 and RFC 2070    */
/*********************************************************************/
#include <stdio.h>
#include "polcnv.h"

long int utf7_hits  = 0;
long int utf7_fails = 0;

/*-------------------------------------------------------------------*/
/* Directly encoded characters					     */
unicode setD[] = 
  {
  0x0027, /* APOSTROPHE */
  0x0028, /* LEFT PARENTHESIS */
  0x0029, /* RIGHT PARENTHESIS */
  0x002C, /* COMMA */
  0x002D, /* HYPHEN-MINUS */
  0x002E, /* FULL STOP */
  0x002F, /* SOLIDUS */
  0x0030, /* DIGIT ZERO */
  0x0031, /* DIGIT ONE */
  0x0032, /* DIGIT TWO */
  0x0033, /* DIGIT THREE */
  0x0034, /* DIGIT FOUR */
  0x0035, /* DIGIT FIVE */
  0x0036, /* DIGIT SIX */
  0x0037, /* DIGIT SEVEN */
  0x0038, /* DIGIT EIGHT */
  0x0039, /* DIGIT NINE */
  0x003A, /* COLON */
  0x003F, /* QUESTION MARK */
  0x0041, /* LATIN CAPITAL LETTER A */
  0x0042, /* LATIN CAPITAL LETTER B */
  0x0043, /* LATIN CAPITAL LETTER C */
  0x0044, /* LATIN CAPITAL LETTER D */
  0x0045, /* LATIN CAPITAL LETTER E */
  0x0046, /* LATIN CAPITAL LETTER F */
  0x0047, /* LATIN CAPITAL LETTER G */
  0x0048, /* LATIN CAPITAL LETTER H */
  0x0049, /* LATIN CAPITAL LETTER I */
  0x004A, /* LATIN CAPITAL LETTER J */
  0x004B, /* LATIN CAPITAL LETTER K */
  0x004C, /* LATIN CAPITAL LETTER L */
  0x004D, /* LATIN CAPITAL LETTER M */
  0x004E, /* LATIN CAPITAL LETTER N */
  0x004F, /* LATIN CAPITAL LETTER O */
  0x0050, /* LATIN CAPITAL LETTER P */
  0x0051, /* LATIN CAPITAL LETTER Q */
  0x0052, /* LATIN CAPITAL LETTER R */
  0x0053, /* LATIN CAPITAL LETTER S */
  0x0054, /* LATIN CAPITAL LETTER T */
  0x0055, /* LATIN CAPITAL LETTER U */
  0x0056, /* LATIN CAPITAL LETTER V */
  0x0057, /* LATIN CAPITAL LETTER W */
  0x0058, /* LATIN CAPITAL LETTER X */
  0x0059, /* LATIN CAPITAL LETTER Y */
  0x005A, /* LATIN CAPITAL LETTER Z */
  0x0061, /* LATIN SMALL LETTER A */
  0x0062, /* LATIN SMALL LETTER B */
  0x0063, /* LATIN SMALL LETTER C */
  0x0064, /* LATIN SMALL LETTER D */
  0x0065, /* LATIN SMALL LETTER E */
  0x0066, /* LATIN SMALL LETTER F */
  0x0067, /* LATIN SMALL LETTER G */
  0x0068, /* LATIN SMALL LETTER H */
  0x0069, /* LATIN SMALL LETTER I */
  0x006A, /* LATIN SMALL LETTER J */
  0x006B, /* LATIN SMALL LETTER K */
  0x006C, /* LATIN SMALL LETTER L */
  0x006D, /* LATIN SMALL LETTER M */
  0x006E, /* LATIN SMALL LETTER N */
  0x006F, /* LATIN SMALL LETTER O */
  0x0070, /* LATIN SMALL LETTER P */
  0x0071, /* LATIN SMALL LETTER Q */
  0x0072, /* LATIN SMALL LETTER R */
  0x0073, /* LATIN SMALL LETTER S */
  0x0074, /* LATIN SMALL LETTER T */
  0x0075, /* LATIN SMALL LETTER U */
  0x0076, /* LATIN SMALL LETTER V */
  0x0077, /* LATIN SMALL LETTER W */
  0x0078, /* LATIN SMALL LETTER X */
  0x0079, /* LATIN SMALL LETTER Y */
  0x007A, /* LATIN SMALL LETTER Z */
  };

/*-------------------------------------------------------------------*/
/* Optional directly encoded characters				     */
unicode setO[] = 
  {
  0x0021, /* EXCLAMATION MARK */
  0x0022, /* QUOTATION MARK */
  0x0023, /* NUMBER SIGN */
  0x0024, /* DOLLAR SIGN */
  0x0025, /* PERCENT SIGN */
  0x0026, /* AMPERSAND */
  0x002A, /* ASTERISK */
  0x003B, /* SEMICOLON */
  0x003C, /* LESS-THAN SIGN */
  0x003D, /* EQUALS SIGN */
  0x003E, /* GREATER-THAN SIGN */
  0x0040, /* COMMERCIAL AT */
  0x005B, /* LEFT SQUARE BRACKET */
  0x005D, /* RIGHT SQUARE BRACKET */
  0x005E, /* CIRCUMFLEX ACCENT */
  0x005F, /* LOW LINE */
  0x0060, /* GRAVE ACCENT */
  0x007B, /* LEFT CURLY BRACKET */
  0x007C, /* VERTICAL LINE */
  0x007D  /* RIGHT CURLY BRACKET */
  };

/*-------------------------------------------------------------------*/
/* Base64 alphabet						     */
unicode setB[] =
  {
  'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
  'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
  'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
  'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
  'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
  'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
  'w', 'x', 'y', 'z', '0', '1', '2', '3',
  '4', '5', '6', '7', '8', '9', '+', '/'
  };

/*-------------------------------------------------------------------*/
/* Static variables for decode					     */
unicode dec_ch0		= 0;	/* Buffer for surrogate pairs	     */
int     dec_escape	= 0;	/* Are we decoding escape sequence?  */
int	dec_bits	= 0;	/* Number of bits used in dec_sbuf   */
unicode	dec_sbuf	= 0;	/* Shift register		     */

/*-------------------------------------------------------------------*/
/* Static variables for encode					     */
int     enc_escape	= 0;	/* Are we emiting escape sequence?   */
int	enc_bits	= 0;	/* Number of bits used in enc_sbuf   */
unicode	enc_sbuf	= 0;	/* Shift register		     */

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int in_set_D( unicode ch )
  {
  int i;
  for (i=0; i<sizeof(setD)/sizeof(unicode); i++)
    {
    if (ch == setD[i])
      {
      return 1;
      }
    }
  return 0;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int in_set_O( unicode ch )
  {
  int i;
  for (i=0; i<sizeof(setO)/sizeof(unicode); i++)
    {
    if (ch == setO[i])
      return 1;
    }
  return 0;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int in_set_B( unicode ch )
  {
  int i;
  for (i=0; i<sizeof(setB)/sizeof(unicode); i++)
    {
    if (ch == setB[i])
      return 1;
    }
  return 0;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int set_B_index( unicode ch )
  {
  int i;
  for (i=0; i<sizeof(setB)/sizeof(unicode); i++)
    {
    if (ch == setB[i])
      return i;
    }
  return -1;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void put_ucs2( unicode dec_ch1 )
  {
  if ((dec_ch0 >= 0x0D800UL) &&
      (dec_ch0 <  0x0DC00UL) &&
      (dec_ch1 >= 0x0DC00UL) &&
      (dec_ch1 <  0x0DF00UL)   )
    {					/* Surrogate pair            */
    uni_buf[ uni_ffree++ ] =
		  ((dec_ch0 - 0x0D800UL) << 10) +
		   (dec_ch1 - 0x0DC00UL) + 0x10000UL;
    dec_ch0 = 0;
    return;
    }
  else if ((dec_ch0 >= 0x0D800UL) &&
           (dec_ch0 <  0x0DC00UL)   )
    {					/* Incomplete surrogate pair */
    if (inp_type != TYPE_TEST)
      fprintf( stderr, "Incomplete UCS-2 surrogate pair %04lX %04lX\n",
						dec_ch0, dec_ch1 );
    uni_buf[ uni_ffree++ ] = dec_ch0;
    dec_ch0 = 0;
    }

  if ((dec_ch1 >= 0x0D800UL) &&
	   (dec_ch1 <  0x0DC00UL)   )
    {					/* Begin of surrogate pair   */
    dec_ch0 = dec_ch1;
    }
  else
    {					/* Just normal character     */
    uni_buf[ uni_ffree++ ] = dec_ch1;
    dec_ch0 = 0;
    }
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void utf7_decode( void )
  {
  int limit;

  limit = inp_ffree;

  while ((inp_fused < inp_ffree) &&
	 (uni_ffree < uni_size ))
    {
    if (dec_escape && in_set_B(inp_buf[inp_fused]))
      {
      dec_sbuf = (dec_sbuf << 6) + set_B_index(inp_buf[inp_fused++]);
      dec_bits+=6;
      if (dec_bits >= 16)
	{
	put_ucs2( (dec_sbuf >> (dec_bits - 16)) & 0x0FFFFUL );
	dec_bits -= 16;
	}
      }
    else if (dec_escape)
      {
      if (inp_buf[inp_fused] == '-')
	inp_fused++;
      dec_escape = FALSE;
      dec_bits = 0;
      dec_sbuf = 0;
      }
    else if (inp_buf[inp_fused] == '+')
      {
      inp_fused++;
      dec_escape = TRUE;
      dec_bits = 0;
      dec_sbuf = 0;
      }
    else if (in_set_D(inp_buf[inp_fused]) || in_set_O(inp_buf[inp_fused]) ||
	inp_buf[inp_fused] == ' '    || inp_buf[inp_fused] == '\t' ||
	inp_buf[inp_fused] == '\n'   || inp_buf[inp_fused] == '\r'  )
      put_ucs2( inp_buf[inp_fused++] );
    else
      {
      if (inp_type != TYPE_TEST)
	fprintf( stderr, "Invalid UTF-7 character %02X\n", inp_buf[inp_fused] );
      inp_fused++;
      }
    }

  if ((inp_ffree == inp_fused) && inp_eof)
    {
    if ((dec_ch0 >= 0x0D800UL) &&
	(dec_ch0 <  0x0DC00UL)   )
      {					/* Incomplete surrogate pair */
      if (inp_type != TYPE_TEST)
	fprintf( stderr, "Incomplete UCS-2 surrogate pair %04lX at EOF\n",
							dec_ch0 );
      uni_buf[ uni_ffree++ ] = dec_ch0;
      dec_ch0 = 0;
      }
    uni_eof = TRUE;
    }
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
unicode get_ucs2( void )
  {
  unicode ch, ch0, ch1;

  if (uni_buf[uni_fused] < 0x0FFFFUL)
    {
    return uni_buf[uni_fused++];
    }
  else if (uni_buf[uni_fused] > 0x10FFFFUL)
    {
    uni_fused++;
    return 0x0FFFDUL;
    }
  else
    {
    ch = uni_buf[uni_fused++] - 0x10000UL;
    ch0 = (ch >> 10) + 0x0D800UL;
    ch1 = (ch & 0x03FFUL) + 0x0DC00UL;
    return (ch0 << 16) + ch1;
    }

  uni_fused++;
  }
/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void utf7_encode( void )
  {
  unicode ch;

  while ((uni_ffree > uni_fused) &&
	 (out_ffree < out_size-5))
    {
    ch = get_ucs2();
    if (in_set_D(ch) ||
	ch == ' '    || ch == '\t' ||
	ch == '\n'   || ch == '\r' ||
	( out_type == TYPE_UNICODE_UTF7_O && in_set_O(ch) ) )
      {				/* Directly encoded characters		*/
      if (enc_escape)
	{
	if (enc_bits != 0)
	  {
	  enc_sbuf <<= (6 - enc_bits);
	  out_buf[out_ffree++] = setB[ enc_sbuf & 0x3F ];
	  }
	out_buf[out_ffree++] = '-';
	enc_sbuf = 0;
	enc_bits = 0;
	enc_escape = FALSE;
	}
      out_buf[out_ffree++] = (byte) ch;
      }
    else
      {
      if (!enc_escape)
	{
	enc_escape = TRUE;
	out_buf[out_ffree++] = '+';
	enc_sbuf = 0;
	enc_bits = 0;
	}
      if (ch >= 0x10000UL)
	{			/* Surrogate pair			*/
	enc_sbuf = (enc_sbuf << 16) + ((ch >> 16) & 0x0FFFFUL);
	enc_bits += 16;
	while (enc_bits - 6 >= 0)
	  {
	  out_buf[out_ffree++] =
		(byte) setB[ (enc_sbuf >> (enc_bits - 6)) & 0x3F ];
	  enc_bits -= 6;
	  }
	ch &= 0x0FFFFUL;
	}
      enc_sbuf = (enc_sbuf << 16) + ch;
      enc_bits += 16;
      while (enc_bits - 6 >= 0)
	{
	out_buf[out_ffree++] =
		(byte) setB[ (enc_sbuf >> (enc_bits - 6)) & 0x3F ];
	enc_bits -= 6;
	}
      }
    }

  if ((uni_ffree == uni_fused) && uni_eof)
    {
    out_eof = TRUE;
    if (enc_escape)
      {
      if (enc_bits != 0)
	{
	enc_sbuf <<= (6 - enc_bits);
	out_buf[out_ffree++] = setB[ enc_sbuf & 0x3F ];
	}
      out_buf[out_ffree++] = '-';
      enc_sbuf = 0;
      enc_bits = 0;
      enc_escape = FALSE;
      }
    }

  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void test_char_utf7( char *buff )
  {
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int test_freq_utf7( void )
  {
  return utf7_hits-utf7_fails;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void utf7_reset_inp( void )
  {
  dec_ch0    = 0;
  dec_escape = FALSE;
  dec_bits   = 0;
  dec_sbuf   = 0;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void utf7_reset_out( void )
  {
  enc_escape = FALSE;
  enc_bits   = 0;
  enc_sbuf   = 0;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init_unicode_utf7( void )
  {
  trans[ TYPE_UNICODE_UTF7 ].key		= "-u7";
  trans[ TYPE_UNICODE_UTF7 ].long_key		= "-utf-7";
  trans[ TYPE_UNICODE_UTF7 ].name		= "Unicode UTF-7";
  trans[ TYPE_UNICODE_UTF7 ].reset_inp		= utf7_reset_inp;
  trans[ TYPE_UNICODE_UTF7 ].reset_out		= utf7_reset_out;
  trans[ TYPE_UNICODE_UTF7 ].decode		= utf7_decode;
  trans[ TYPE_UNICODE_UTF7 ].encode		= utf7_encode;
  trans[ TYPE_UNICODE_UTF7 ].inp_tbl		= NULL;
  trans[ TYPE_UNICODE_UTF7 ].inp_tbl_size	= 0;
  trans[ TYPE_UNICODE_UTF7 ].out_tbl		= NULL;
  trans[ TYPE_UNICODE_UTF7 ].out_tbl_size	= 0;
  trans[ TYPE_UNICODE_UTF7 ].buf_tst		= NULL;
  trans[ TYPE_UNICODE_UTF7 ].freq_tst		= test_freq_utf7;

  trans[ TYPE_UNICODE_UTF7_O ].key		= "-u7o";
  trans[ TYPE_UNICODE_UTF7_O ].long_key		= "-utf-7-opt";
  trans[ TYPE_UNICODE_UTF7_O ].name		=
		"Unicode UTF-7 with optional characters encoded directly";
  trans[ TYPE_UNICODE_UTF7_O ].reset_inp	= utf7_reset_inp;
  trans[ TYPE_UNICODE_UTF7_O ].reset_out	= utf7_reset_out;
  trans[ TYPE_UNICODE_UTF7_O ].decode		= utf7_decode;
  trans[ TYPE_UNICODE_UTF7_O ].encode		= utf7_encode;
  trans[ TYPE_UNICODE_UTF7_O ].inp_tbl		= NULL;
  trans[ TYPE_UNICODE_UTF7_O ].inp_tbl_size	= 0;
  trans[ TYPE_UNICODE_UTF7_O ].out_tbl		= NULL;
  trans[ TYPE_UNICODE_UTF7_O ].out_tbl_size	= 0;
  trans[ TYPE_UNICODE_UTF7_O ].buf_tst		= NULL;
  trans[ TYPE_UNICODE_UTF7_O ].freq_tst		= test_freq_utf7;
  }

/*********************************************************************/
