/*********************************************************************/
/* utf8.c    v 2.0      by J.Sobczyk IAPW       1997.09.04  21:00    */
/*                                                                   */
/* Polish document converter.                                        */
/* Unicode (ISO 10646) UTF8 as described in RFC 2044                 */
/*********************************************************************/
#include <stdio.h>
#include "polcnv.h"

long int utf8_hits  = 0;
long int utf8_fails = 0;

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void utf8_decode( void )
  {
  int limit;
  int i;
  int len;

  limit = inp_ffree;

  while ((inp_fused < limit) &&
	 (uni_ffree < uni_size ))
    {
    if ((inp_buf[inp_fused] & 0x0FE) == 0x0FC)
      {
      uni_buf[ uni_ffree ] = (unicode)(inp_buf[ inp_fused++ ] & 0x001);
      len=5;
      }
    else if ((inp_buf[inp_fused] & 0x0FC) == 0x0F8)
      {
      uni_buf[ uni_ffree ] = (unicode)(inp_buf[ inp_fused++ ] & 0x003);
      len=4;
      }
    else if ((inp_buf[inp_fused] & 0x0F8) == 0x0F0)
      {
      uni_buf[ uni_ffree ] = (unicode)(inp_buf[ inp_fused++ ] & 0x007);
      len=3;
      }
    else if ((inp_buf[inp_fused] & 0x0F0) == 0x0E0)
      {
      uni_buf[ uni_ffree ] = (unicode)(inp_buf[ inp_fused++ ] & 0x00F);
      len=2;
      }
    else if ((inp_buf[inp_fused] & 0x0E0) == 0x0C0)
      {
      uni_buf[ uni_ffree ] = (unicode)(inp_buf[ inp_fused++ ] & 0x01F);
      len=1;
      }
    else if ((inp_buf[inp_fused] & 0x0C0) == 0x080)
      {
      if (inp_type != TYPE_TEST)
	fprintf( stderr, "Invalid UTF-8 sequence starting with: 0x%0X\n",
						 inp_buf[inp_fused] );
      }
    else 
      {
      uni_buf[ uni_ffree ] = (unicode)(inp_buf[ inp_fused++ ] & 0x07F);
      len=0;
      }

    if ((uni_ffree+len >= uni_size) && !inp_eof)
      {
      inp_fused--;
      break;
      }
    else if ((uni_ffree+len >= uni_size) && inp_eof)
      {
      len = uni_size - uni_ffree;
      if (inp_type != TYPE_TEST)
	fprintf( stderr, "Incomplete UTF-8 sequence at the end of file\n" );
      }

    for (i=0; i<len; i++)
      {
      uni_buf[ uni_ffree ] <<= 6;
      uni_buf[ uni_ffree ] |= (unicode)(inp_buf[ inp_fused++ ] & 0x03F);
      }
    uni_ffree++;
    }

  if ((inp_ffree == inp_fused) && inp_eof)
    uni_eof = TRUE;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void utf8_encode( void )
  {
  while ((uni_ffree > uni_fused) &&
	 (out_ffree < out_size-5))
    {
    if ((uni_buf[uni_fused] & 0xFFFFFF80) == 0)
      {
      out_buf[out_ffree++] = (byte) uni_buf[uni_fused];
      }
    else if ((uni_buf[uni_fused] & 0xFFFFF800) == 0)
      {
      out_buf[out_ffree++] = (byte) ((uni_buf[uni_fused] >> 6)  & 0x01F) | 0x0C0;
      out_buf[out_ffree++] = (byte) ( uni_buf[uni_fused]        & 0x03F) | 0x080;
      }
    else if ((uni_buf[uni_fused] & 0xFFFFF800) == 0)
      {
      out_buf[out_ffree++] = (byte) ((uni_buf[uni_fused] >> 6)  & 0x01F) | 0x0C0;
      out_buf[out_ffree++] = (byte) ( uni_buf[uni_fused]        & 0x03F) | 0x080;
      }
    else if ((uni_buf[uni_fused] & 0xFFFF0000) == 0)
      {
      out_buf[out_ffree++] = (byte) ((uni_buf[uni_fused] >> 12) & 0x00F) | 0x0E0;
      out_buf[out_ffree++] = (byte) ((uni_buf[uni_fused] >> 6)  & 0x03F) | 0x080;
      out_buf[out_ffree++] = (byte) ( uni_buf[uni_fused]        & 0x03F) | 0x080;
      }
    else if ((uni_buf[uni_fused] & 0xFFE00000) == 0)
      {
      out_buf[out_ffree++] = (byte) ((uni_buf[uni_fused] >> 18) & 0x007) | 0x0F0;
      out_buf[out_ffree++] = (byte) ((uni_buf[uni_fused] >> 12) & 0x03F) | 0x080;
      out_buf[out_ffree++] = (byte) ((uni_buf[uni_fused] >> 6)  & 0x03F) | 0x080;
      out_buf[out_ffree++] = (byte) ( uni_buf[uni_fused]        & 0x03F) | 0x080;
      }
    else if ((uni_buf[uni_fused] & 0xFC000000) == 0)
      {
      out_buf[out_ffree++] = (byte) ((uni_buf[uni_fused] >> 24) & 0x003) | 0x0F8;
      out_buf[out_ffree++] = (byte) ((uni_buf[uni_fused] >> 18) & 0x03F) | 0x080;
      out_buf[out_ffree++] = (byte) ((uni_buf[uni_fused] >> 12) & 0x03F) | 0x080;
      out_buf[out_ffree++] = (byte) ((uni_buf[uni_fused] >> 6)  & 0x03F) | 0x080;
      out_buf[out_ffree++] = (byte) ( uni_buf[uni_fused]        & 0x03F) | 0x080;
      }
    else if ((uni_buf[uni_fused] & 0x80000000) == 0)
      {
      out_buf[out_ffree++] = (byte) ((uni_buf[uni_fused] >> 30) & 0x001) | 0x0FC;
      out_buf[out_ffree++] = (byte) ((uni_buf[uni_fused] >> 24) & 0x03F) | 0x080;
      out_buf[out_ffree++] = (byte) ((uni_buf[uni_fused] >> 18) & 0x03F) | 0x080;
      out_buf[out_ffree++] = (byte) ((uni_buf[uni_fused] >> 12) & 0x03F) | 0x080;
      out_buf[out_ffree++] = (byte) ((uni_buf[uni_fused] >> 6)  & 0x03F) | 0x080;
      out_buf[out_ffree++] = (byte) ( uni_buf[uni_fused]        & 0x03F) | 0x080;
      }
    else 
      {
      if (inp_type != TYPE_TEST)
	fprintf( stderr, "Invalid UCS-4 character: 0x%0lX\n",
						 uni_buf[uni_fused] );
      }

    uni_fused++;
    }

  if ((uni_ffree == uni_fused) && uni_eof)
    out_eof = TRUE;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void test_char_utf8( char *buff )
  {
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
int test_freq_utf8( void )
  {
  return utf8_hits-utf8_fails;
  }

/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
void init_unicode_utf8( void )
  {
  trans[ TYPE_UNICODE_UTF8 ].key		= "-u8";
  trans[ TYPE_UNICODE_UTF8 ].long_key		= "-utf-8";
  trans[ TYPE_UNICODE_UTF8 ].name		= "Unicode UTF-8";
  trans[ TYPE_UNICODE_UTF8 ].reset_inp		= NULL;
  trans[ TYPE_UNICODE_UTF8 ].reset_out		= NULL;
  trans[ TYPE_UNICODE_UTF8 ].decode		= utf8_decode;
  trans[ TYPE_UNICODE_UTF8 ].encode		= utf8_encode;
  trans[ TYPE_UNICODE_UTF8 ].inp_tbl		= NULL;
  trans[ TYPE_UNICODE_UTF8 ].inp_tbl_size	= 0;
  trans[ TYPE_UNICODE_UTF8 ].out_tbl		= NULL;
  trans[ TYPE_UNICODE_UTF8 ].out_tbl_size	= 0;
  trans[ TYPE_UNICODE_UTF8 ].buf_tst		= NULL;
  trans[ TYPE_UNICODE_UTF8 ].freq_tst		= test_freq_utf8;
  }

/*********************************************************************/
